@@ -22,7 +22,7 @@
  * along with mx-repo-manager.  If not, see <http://www.gnu.org/licenses/>.
  **********************************************************************/
 
-
+#include "about.h"
 #include "mainwindow.h"
 #include "ui_mainwindow.h"
 #include "version.h"
@@ -35,12 +35,11 @@
 #include <QProgressBar>
 #include <QTextEdit>
 
-
 MainWindow::MainWindow(QWidget *parent) :
     QDialog(parent),
     ui(new Ui::MainWindow)
 {
-    qDebug() << "Program Version:" << VERSION;
+    qDebug().noquote() << QCoreApplication::applicationName() << "version:" << VERSION;
     ui->setupUi(this);
     if (ui->buttonOk->icon().isNull()) {
         ui->buttonOk->setIcon(QIcon(":/icons/dialog-ok.svg"));
@@ -53,15 +52,18 @@ MainWindow::MainWindow(QWidget *parent) :
     }
 
     shell = new Cmd(this);
+
     connect(shell, &Cmd::started, this, &MainWindow::procStart);
     connect(shell, &Cmd::finished, this, &MainWindow::procDone);
-    connect(shell, &Cmd::runTime, this, &MainWindow::procTime);
 
     progress = new QProgressDialog(this);
     bar = new QProgressBar(progress);
+    bar->setMaximum(100);
+    progCancel = new QPushButton(tr("Cancel"));
+    connect(progCancel, &QPushButton::clicked, this, &MainWindow::cancelOperation);
     progress->setWindowModality(Qt::WindowModal);
     progress->setWindowFlags(Qt::Dialog | Qt::CustomizeWindowHint | Qt::WindowTitleHint |Qt::WindowSystemMenuHint | Qt::WindowStaysOnTopHint);
-    progress->setCancelButton(0);
+    progress->setCancelButton(progCancel);
     progress->setLabelText(tr("Please wait..."));
     progress->setAutoClose(false);
     progress->setBar(bar);
@@ -156,12 +158,12 @@ QStringList MainWindow::readantiXRepos()
 // List current repo
 void MainWindow::getCurrentRepo()
 {
-    current_repo  = shell->getOutput("grep -m1 '^deb.*/repo/ ' /etc/apt/sources.list.d/antix.list | cut -d' ' -f2 | cut -d/ -f3");
+    current_repo  = shell->getCmdOut("grep -m1 '^deb.*/repo/ ' /etc/apt/sources.list.d/antix.list | cut -d' ' -f2 | cut -d/ -f3");
 }
 
 QString MainWindow::getDebianVersion()
 {
-    return shell->getOutput("cat /etc/debian_version | cut -f1 -d'.'");
+    return shell->getCmdOut("cat /etc/debian_version | cut -f1 -d'.'");
 }
 
 // display available repos
@@ -252,10 +254,16 @@ void MainWindow::displayAllRepos(const QFileInfoList &apt_files)
 
 QStringList MainWindow::loadAptFile(const QString &file)
 {
-    QString entries = shell->getOutput("grep '^#*[ ]*deb' " + file);
+    QString entries = shell->getCmdOut("grep '^#*[ ]*deb' " + file);
     return entries.split("\n");
 }
 
+void MainWindow::cancelOperation()
+{
+    shell->halt();
+    procDone();
+}
+
 // displays the current repo by selecting the item
 void MainWindow::displaySelected(const QString &repo)
 {
@@ -297,18 +305,22 @@ void MainWindow::procTime()
         bar->reset();
     }
     bar->setValue(bar->value() + 1);
-    //qApp->processEvents();
+//    qApp->processEvents();
 }
 
 void MainWindow::procStart()
 {
     bar->setValue(0);
     QApplication::setOverrideCursor(QCursor(Qt::BusyCursor));
+    timer.start(100);
+    connect(&timer, &QTimer::timeout, this, &MainWindow::procTime);
 }
 
 void MainWindow::procDone()
 {
     bar->setValue(100);
+    timer.stop();
+    timer.disconnect();
     QApplication::setOverrideCursor(QCursor(Qt::ArrowCursor));
 }
 
@@ -340,7 +352,7 @@ void MainWindow::replaceRepos(const QString &url)
     cmd_antix = QString("sed -i 's;https\\?://.*/" + ver_name + "/\\?;%1;' %2").arg(repo_line_antix).arg(antix_file);
 
     // check if both replacement were successful
-    if (shell->run(cmd_antix) == 0 && shell->run(cmd_antix) == 0) {
+    if (shell->run(cmd_antix) && shell->run(cmd_antix)) {
         QMessageBox::information(this, tr("Success"),
                                  tr("Your new selection will take effect the next time sources are updated."));
     } else {
@@ -387,40 +399,12 @@ void MainWindow::on_buttonOk_clicked()
 void MainWindow::on_buttonAbout_clicked()
 {
     this->hide();
-    QMessageBox msgBox(QMessageBox::NoIcon,
-                       tr("About Repo Manager"), "<p align=\"center\"><b><h2>" +
-                       tr("Repo Manager") + "</h2></b></p><p align=\"center\">" + tr("Version: ") + version + "</p><p align=\"center\"><h3>" +
+    displayAboutMsgBox(tr("About %1").arg(this->windowTitle()), "<p align=\"center\"><b><h2>" + this->windowTitle() +"</h2></b></p><p align=\"center\">" +
+                       tr("Version: ") + VERSION + "</p><p align=\"center\"><h3>" +
                        tr("Program for choosing the default APT repository") +
                        "</h3></p><p align=\"center\"><a href=\"http://mxlinux.org\">http://mxlinux.org</a><br /></p><p align=\"center\">" +
-                       tr("Copyright (c) MX Linux") + "<br /><br /></p>", 0, this);
-    QPushButton *btnLicense = msgBox.addButton(tr("License"), QMessageBox::HelpRole);
-    QPushButton *btnChangelog = msgBox.addButton(tr("Changelog"), QMessageBox::HelpRole);
-    QPushButton *btnCancel = msgBox.addButton(tr("Cancel"), QMessageBox::NoRole);
-    btnCancel->setIcon(QIcon::fromTheme("window-close"));
-
-    msgBox.exec();
-
-    if (msgBox.clickedButton() == btnLicense) {
-        QString url = "file:///usr/share/doc/mx-repo-manager/license.html";
-        displayDoc(url);
-    } else if (msgBox.clickedButton() == btnChangelog) {
-        QDialog *changelog = new QDialog(this);
-        changelog->resize(600, 500);
-
-        QTextEdit *text = new QTextEdit;
-        text->setReadOnly(true);
-        text->setText(shell->getOutput("zless /usr/share/doc/" + QFileInfo(QCoreApplication::applicationFilePath()).fileName()  + "/changelog.gz"));
-
-        QPushButton *btnClose = new QPushButton(tr("&Close"));
-        btnClose->setIcon(QIcon::fromTheme("window-close"));
-        connect(btnClose, &QPushButton::clicked, changelog, &QDialog::close);
-
-        QVBoxLayout *layout = new QVBoxLayout;
-        layout->addWidget(text);
-        layout->addWidget(btnClose);
-        changelog->setLayout(layout);
-        changelog->exec();
-    }
+                       tr("Copyright (c) MX Linux") + "<br /><br /></p>",
+                       "/usr/share/doc/repo-manager/license.html", tr("%1 License").arg(this->windowTitle()), true);
     this->show();
 }
 
@@ -430,12 +414,12 @@ void MainWindow::on_buttonHelp_clicked()
     QLocale locale;
     QString lang = locale.bcp47Name();
 
-    QString url = "https://mxlinux.org/wiki/help-files/help-mx-repo-manager";
+    QString url = "/usr/share/doc/repo-manager/help/mx-repo-manager.html";
 
     if (lang.startsWith("fr")) {
         url = "https://mxlinux.org/wiki/help-files/help-mx-gestionnaire-de-d%C3%A9p%C3%B4ts";
     }
-    displayDoc(url);
+    displayDoc(url, tr("%1 Help").arg(this->windowTitle()), true);
 }
 
 void MainWindow::on_treeWidget_itemChanged(QTreeWidgetItem * item, int column)
@@ -529,24 +513,13 @@ QIcon MainWindow::getFlag(QString country)
     return QIcon();
 }
 
-void MainWindow::displayDoc(QString url)
-{
-    QString exec = "xdg-open";
-    QString user = shell->getOutput("logname");
-    if (system("command -v antix-viewer") == 0) { // use antix-viewer if available
-        exec = "antix-viewer";
-    }
-    QString cmd = "su " + user + " -c \"" + exec + " " + url + "\"&";
-    system(cmd.toUtf8());
-}
-
 // detect fastest Debian repo
 void MainWindow::on_pushFastestDebian_clicked()
 {
     QString repo;
 
     progress->show();
-    QString tmpfile = shell->getOutput("mktemp -d /tmp/repo-manager-XXXXXXXX") + "/sources.list";
+    QString tmpfile = shell->getCmdOut("mktemp -d /tmp/repo-manager-XXXXXXXX") + "/sources.list";
 
     QString ver_num = getDebianVersion();
     QString ver_name;
@@ -556,18 +529,19 @@ void MainWindow::on_pushFastestDebian_clicked()
         ver_name = "stretch";
     }
 
-    QString out = shell->getOutput("netselect-apt " + ver_name + " -o " + tmpfile);
+    QByteArray out;
+    bool success = shell->run("netselect-apt " + ver_name + " -o " + tmpfile, out, false);
     progress->hide();
 
-    if (shell->getExitCode(true) != 0) {
+    if (!success) {
         QMessageBox::critical(this, tr("Error"),
                               tr("netselect-apt could not detect fastest repo."));
         return;
     }
-    repo = shell->getOutput("set -o pipefail; grep -m1 '^deb ' " + tmpfile + "| cut -d' ' -f2");
+    repo = shell->getCmdOut("set -o pipefail; grep -m1 '^deb ' " + tmpfile + "| cut -d' ' -f2");
     this->blockSignals(false);
 
-    if (shell->getExitCode(true) == 0 && shell->run("wget --spider " + repo) == 0) {
+    if (success && shell->run("wget --spider " + repo)) {
         replaceDebianRepos(repo);
         refresh();
     } else {
@@ -580,11 +554,12 @@ void MainWindow::on_pushFastestDebian_clicked()
 void MainWindow::on_pushFastestantiX_clicked()
 {
     progress->show();
-    QString out = shell->getOutput("set -o pipefail; netselect -D -I " + listantiXurls + "| tr -s ' ' | sed 's/^ //' | cut -d' ' -f2");
+    QByteArray out;
+    bool success = shell->run("set -o pipefail; netselect -D -I " + listantiXurls + "| tr -s ' ' | sed 's/^ //' | cut -d' ' -f2", out);
     qDebug() << listantiXurls;
-    qDebug() << "FASTEST " << shell->getExitCode(true) << out;
+    qDebug() << "FASTEST " << success << out;
     progress->hide();
-    if (shell->getExitCode(true) == 0 && !out.isEmpty()) {
+    if (success && !out.isEmpty()) {
         displaySelected(out);
         on_buttonOk_clicked();
     } else {
@@ -606,16 +581,16 @@ void MainWindow::on_lineSearch_textChanged(const QString &arg1)
 
 void MainWindow::on_pb_restoreSources_clicked()
 {
-    QString antix_version = shell->getOutput("lsb_release -rs").left(2);
+    QString antix_version = shell->getCmdOut("grep -oP '(?<=DISTRIB_RELEASE=).*' /etc/lsb-release").left(2);
     if (antix_version.toInt() < 15) {
         qDebug() << "antiX version not detected or out of range: " << antix_version;
         return;
     }
     // create temp folder
-    QString path = shell->getOutput("mktemp -d /tmp/antix-sources.XXXXXX");
+    QString path = shell->getCmdOut("mktemp -d /tmp/antix-sources.XXXXXX");
     // download source files from
-    QString cmd = QString("wget -q https://github.com/antix-linux/antiX-%1_sources/archive/master.zip -P %2").arg(antix_version).arg(path);
-    if (shell->run(cmd.toUtf8()) != 0) {
+    QString cmd = QString("wget -q https://gitlab.com/mx-linux/MX-%1_sources/archive/master.zip -P %2").arg(antix_version).arg(path);
+    if (!shell->run(cmd.toUtf8())) {
         QMessageBox::critical(this, tr("Error"),
                               tr("Could not download original APT files."));
         return;
